@@ -5,12 +5,13 @@ import subprocess
 import glob
 from thornpy.signal import step_function, low_pass
 from numpy import linspace, argmax, array
-from ..postprocess.xml import get_results
+from ..postprocess.xml import get_results, shrink_results
 from .utilities import build, add_splines_to_acf, add_splines_to_adm
 from ..postprocess import launch_ppt
 from .string import DrillString
 from .event import DrillEvent
 from .solver_settings import DrillSolverSettings
+from adamspy.adamspy import get_simdur_from_acf, get_simdur_from_msg
 
 class DrillSim(): #pylint: disable=too-many-instance-attributes
     """Contains data defining the files that make up an Adams Drill input deck.    
@@ -54,8 +55,12 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         Simulation results
     results_units : dict
         Units of simulation results
+    pason_inputs : dict
+        Contains the cleaned pason signals that will be used as inputs to the Adams model. Keys are 'time', 'rop', 'wob', 'rpm', and 'gpm'.  Each value is a :obj:`list` with two entries.  The first is the signal and the second is the associated time.
     built : bool
         Indicates whether the input deck (adm, acf, and cmd files) has been built yet for this DrillSim
+    solved : bool
+        Indicates whether the :class:`DrillSim` has been solved and results file (.res) has been generated.
     RAMP_TIME : dict
         Class attribute containing standard ramp times for rpm, gpm, wob, and rop
     CUTOFF_FREQ : dict
@@ -102,6 +107,7 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
 
         # Flags
         self.built = False
+        self.solved = False
 
     @classmethod
     def read_from_directory(cls, directory):
@@ -143,26 +149,30 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         drill_sim = cls(string, event, solver_settings, directory, analysis_name, write_TO_files=False)
 
         # Define file names
-        drill_sim.string_filename = os.path.split(string_filename)[1]   
+        drill_sim.string_filename = string_filename
         
         adm_files = glob.glob(os.path.join(directory, '*.adm'))
-        drill_sim.adm_filename = os.path.split(adm_files[0])[1] if adm_files else ''
+        drill_sim.adm_filename = adm_files[0] if adm_files else ''
 
         acf_files = glob.glob(os.path.join(directory, '*.acf'))
-        drill_sim.acf_filename = os.path.split(acf_files[0])[1] if acf_files else ''
+        drill_sim.acf_filename = acf_files[0] if acf_files else ''
 
         cmd_files = glob.glob(os.path.join(directory, '*.cmd'))
-        drill_sim.cmd_filename = os.path.split(cmd_files[0])[1] if cmd_files else ''
+        drill_sim.cmd_filename = cmd_files[0] if cmd_files else ''
 
         res_files = glob.glob(os.path.join(directory, '*.res'))
-        drill_sim.res_filename = os.path.split(res_files[0])[1] if res_files else ''
+        drill_sim.res_filename = res_files[0] if res_files else ''
 
         msg_files = glob.glob(os.path.join(directory, '*.msg'))
-        drill_sim.msg_filename = os.path.split(msg_files[0])[1] if msg_files else ''
+        drill_sim.msg_filename =msg_files[0] if msg_files else ''
 
         # Set the `built` flag
         if drill_sim.acf_filename and drill_sim.adm_filename:            
             drill_sim.built = True
+        
+        # set the `solved` flag
+        if drill_sim.res_filename:
+            drill_sim.solved = True
 
         return drill_sim
 
@@ -197,11 +207,17 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         # Add the time signal
         self.pason_inputs['time'] = list(pason_data.data.index - pason_data.data.index[i_min])
 
-    def build(self):
+    def build(self, wait=True):
         """This method builds the input deck.  It launches Adams View in batch, reads in the `self.string.filename` and `self.solver_settings.filename` files, and runs the Adams Drill macro ``ds tostart`` to build a drill string model.  Then it saves the model files (.acf, .adm, and .cmd) to the `self.directory` directory.
+
+        Parameters
+        ----------
+        wait : bool
+            If True, code execution waits until the build process terminates before moving on. (default is True)
+
         """
         # Build the model
-        adm, acf, cmd = build(self.string_filename, self.solver_settings.filename, self.directory)  
+        adm, acf, cmd = build(self.string_filename, self.solver_settings.filename, self.directory, wait=wait)  
 
         # store the new filenames as attributes
         self.adm_filename = adm
@@ -226,7 +242,7 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         """                
         startupinfo = subprocess.STARTUPINFO()
         startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW        
-        self.run_proc = subprocess.Popen('{} ru-s {}'.format(os.environ['ADAMS_LAUNCH_COMMAND'], os.path.split(self.acf_filename)[-1]), cwd=self.directory, startupinfo=startupinfo)
+        self.run_proc = subprocess.Popen('"{}" ru-s "{}"'.format(os.environ['ADAMS_LAUNCH_COMMAND'], os.path.split(self.acf_filename)[-1]), cwd=self.directory, startupinfo=startupinfo)
         
         # Wait for the process to complete before moving on.
         if wait:
@@ -234,6 +250,9 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
 
         self.msg_filename = f'{self.analysis_name}.msg'
         self.res_filename = f'{self.analysis_name}.res'
+
+        # Flag this simulation as solved
+        self.solved = True
     
     def write_tiem_orbit_files(self):
         """Writes the solver settings and event files and publishes the string file to the simulation directory.
@@ -255,7 +274,7 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         self.string.parameters['OutputName'] = self.analysis_name
         self.string_filename = self.string.write_to_file(directory=self.directory, publish=True)   
     
-    def read_results(self, reqs_to_read=None, t_min=None, t_max=None):
+    def read_results(self, reqs_to_read=None, t_min=None, t_max=None, shrink_results_file=False):
         """Reads results from the results file.
         
         Example
@@ -278,11 +297,17 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
             Mininmum time for which to get results (the default is None, which gets results starting at the first time step)
         t_max : float, optional
             Maximum time for which to get results (the default is None, which gets results up to the last time step)
+        shrink_results_file : bool, optional
+            If True, the results file will be rewritten to include only the requests in `reqs_to_read` and the time period between `t_max` and `t_min`.
         
         """
         res_file = os.path.join(self.directory, self.res_filename)
         self.results, self.results_units = get_results(res_file, reqs_to_read, t_min, t_max, return_units=True)
 
+        if shrink_results_file and any([reqs_to_read is not None, t_min is not None, t_max is not None]):
+            shrink_results(res_file, reqs_to_keep=reqs_to_read, t_min=t_min, t_max=t_max, in_place=True)
+
+
     def launch_ppt(self, wait=False):
         """Launches the postprocessor and loads :attr:`res_filename`
         
@@ -385,8 +410,6 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
             A :class:`PasonData` object
         sig_type : str
             Type of signal. Options are 'wob' or 'rop'
-        show_plot : bool, optional
-            [description] (the default is True, which [default_description])
         cutoff_freq : float, optional
             Cutoff frequency to be used in the lowpass filter. (the default is None, which uses value from PasonData.CUTOFF_FREQ)
         
@@ -475,6 +498,29 @@ class DrillSim(): #pylint: disable=too-many-instance-attributes
         signal.extend([set_points[-1][1] for t in time_seg])
         time.extend(time_seg)
         return signal, time
+    
+    def get_duration(self, use_acf=False):
+        """Returns the duration of the :class:`DrillSim` simulation based on contents of the acf file or the msg file.
+        
+        Parameters
+        ----------
+        use_acf : bool (optional)
+            Set this to `True` if you want to use the acf file even if the :class:`DrillSim` has been solved. (the default is `False`)
+
+        Returns
+        -------
+        float
+            Duration of the simulation
+
+        """
+        if self.solved and not use_acf:
+            duration = get_simdur_from_msg(os.path.join(self.directory, self.msg_filename))
+        elif self.built:
+            duration = get_simdur_from_acf(os.path.join(self.directory, self.acf_filename))
+        else:
+            duration = None
+        
+        return duration        
 
     def _add_adm_splines(self):
         """Adds splines to the adm file
