@@ -2,6 +2,7 @@
 """
 import os
 import copy
+import thornpy
 from . import TMPLT_ENV
 from .utilities import read_TO_file, get_cdb_path, get_full_path
 
@@ -127,7 +128,7 @@ class DrillSolverSettings():
 
         elif filename is not None:
             # If Nothing but a filename is given, set that as the full path
-            filepath = os.path.normpath(filename.replace(f'.{self._EXT}',''))            
+            filepath = thornpy.utilities.convert_path(filename.replace(f'.{self._EXT}',''))            
 
         else:
             # If nothing is given, raise an error
@@ -227,7 +228,7 @@ class DrillSolverSettings():
         """
 
         for param in self._TABLE_PARAMETERS:
-            # For each string parameter initialize a found flag
+            # For each parameter initialize a found flag
             found = False
             
             if param.lower() == 'funnel':
@@ -241,7 +242,7 @@ class DrillSolverSettings():
                 raise ValueError(f'{param} not found!')
 
         for param in self._SCALAR_PARAMETERS:
-            # For each string parameter initialize a found flag
+            # For each parameter initialize a found flag
             found = False
 
             for block in tiem_orbit_data:
