@@ -112,6 +112,16 @@ class Test_ReadResults(unittest.TestCase):
         # Reset the ADRILL_USER_CFG environment variable
         os.environ['ADRILL_USER_CFG'] = os.path.join(os.environ['USERPROFILE'], '.adrill.cfg')
 
+class Test_DrillSimFromExistingEventFile(unittest.TestCase):
+
+    def setUp(self):        
+        return
+        
+        self.assertEqual(0,1)
+
+    def tearDown(self):
+        return
+
 class Test_DrillSim(unittest.TestCase):
     """Tests related to the DrillSim class
     """
@@ -150,9 +160,9 @@ class Test_DrillSim(unittest.TestCase):
         self.event = adripy.DrillEvent(TEST_EVENT_NAME,2000, 3)
         self.event.add_simulation_step(10)
         self.event.add_simulation_step(100)
-        self.event.add_ramp('PUMP_FLOW', 0, 15, 500)
-        self.event.add_ramp('TOP_DRIVE', 15, 15, 60)
-        self.event.add_ramp('WOB', 30, 15, 50)
+        self.event.add_ramp('FLOW_RATE', 0, 15, 500)
+        self.event.add_ramp('ROTARY_RPM', 15, 15, 60)
+        self.event.add_ramp('WOB', 30, 15, 50000)
         self.event.add_ramp('ROP', 30, 15, 100)
 
         # Create a solver settings object
@@ -162,6 +172,22 @@ class Test_DrillSim(unittest.TestCase):
         if drill_sim.built is False:
             drill_sim.build()
 
+    def test_drillsim_from_existing_event_file_2018(self):
+        event_from_file = adripy.event.DrillEvent.read_from_file(TEST_EVENT_FILE)
+        drill_sim = DrillSim(self.drill_string, event_from_file, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
+        drill_sim.build()
+
+        failures = check_file_contents(drill_sim.acf_filename, EXPECTED_DRILLSIM_FROM_EXISTING_EVENT_FILE_ACF_TEXT)
+        self.assertListEqual(failures, [])
+
+    def test_drillsim_from_existing_event_file_2019(self):
+        event_from_file = adripy.event.DrillEvent.read_from_file(TEST_EVENT_FILE_2019_2)
+        drill_sim = DrillSim(self.drill_string, event_from_file, self.solver_settings, TEST_WORKING_DIRECTORY, TEST_ANALYSIS_NAME)
+        drill_sim.build()
+
+        failures = check_file_contents(drill_sim.acf_filename, EXPECTED_DRILLSIM_FROM_EXISTING_EVENT_FILE_ACF_TEXT)
+        self.assertListEqual(failures, [])
+
     def test_string_file_contents(self):
         """Checks that the contents of the string file generated by drill_sim.setup() are as expected.
         
